﻿using Plane.Copters;
using Plane;
using Plane.FormationCreator.Formation;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.CommandWpf;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Threading;
using Plane.Communication;
using Plane.Geography;

namespace Plane.FormationCreator.ViewModels
{
    public class CopterListViewModel : ViewModelBase
    {
        public CopterListViewModel(CopterManager copterManager, MapManager mapManager, FlightTaskManager flightTaskManager)
        {
            _copterManager = copterManager;
            _mapManager = mapManager;
            _flightTaskManager = flightTaskManager;
        }

        private CopterManager _copterManager;
        private MapManager _mapManager;
        private FlightTaskManager _flightTaskManager;

        public CopterManager CopterManager { get { return _copterManager; } }

        private ICopter _SelectedCopter;
        public ICopter SelectedCopter
        {
            get { return _SelectedCopter; }
            set { Set(nameof(SelectedCopter), ref _SelectedCopter, value); }
        }

        private int _virtualCopterId = 1;
        private LatLng? _lastVirtualCopterLocation;


        private ICommand _AddVirtualCopterCommand;
        public ICommand AddVirtualCopterCommand
        {
            get
            {
                return _AddVirtualCopterCommand ?? (_AddVirtualCopterCommand = new RelayCommand<int>(async addcount =>
                {
                    var center = _mapManager.Center;
                    string id;

                    int colnum = 5; //自动生成列数=4
                    float coldis = 5;//列相距5米
                    float rowdis = 5;//行相距5米
                    int currcol = 0; //当前列号
                    int currrow = 0; //当前行
                    Tuple<double, double> colheadLatLng = new Tuple<double, double>(0, 0);
                    Tuple<double, double> targetLatLng = new Tuple<double, double>(0, 0);

                    LatLng? colLatLng = new LatLng(center.Lat, center.Lng); ;

                    for (int i = 0; i < addcount; ++i, ++_virtualCopterId)
                    {
                        id = _virtualCopterId.ToString();


 
                        if (i == 0)
                        {
                            _lastVirtualCopterLocation = new LatLng(center.Lat, center.Lng);

                            colLatLng =  _lastVirtualCopterLocation;



                        }
                        else
                        {
                            if (currcol < colnum)
                                targetLatLng = GeographyUtils.CalcLatLngSomeMetersAway2D(colLatLng.Value.Lat, colLatLng.Value.Lng , 90, currcol * coldis);
                            else
                            {
                                currrow++;
                                currcol = 0;
                                targetLatLng = GeographyUtils.CalcLatLngSomeMetersAway2D(colLatLng.Value.Lat, colLatLng.Value.Lng, 180, rowdis);
                                colLatLng = new LatLng(targetLatLng.Item1, targetLatLng.Item2); 

                                colheadLatLng = targetLatLng;
                            }
                            _lastVirtualCopterLocation= new LatLng(targetLatLng.Item1, targetLatLng.Item2);
                        }
                        currcol++;
   



                      /*  _lastVirtualCopterLocation =
                            _lastVirtualCopterLocation == null ?
                            new LatLng(center.Lat, center.Lng) :
                            new LatLng(_lastVirtualCopterLocation.Value.Lat, _lastVirtualCopterLocation.Value.Lng + 0.0001);
                            */















                        var copter = new FakeCopter(SynchronizationContext.Current);
                        copter.SetProperties(
                            latitude: _lastVirtualCopterLocation.Value.Lat,
                            longitude: _lastVirtualCopterLocation.Value.Lng,
                            id: id,
                            name: id
                        );
                        await copter.ConnectAsync();
                        await copter.GetCopterDataAsync();
                        _copterManager.Copters.Add(copter);
                        _copterManager.CopterStatus.Add(false);
                    }
                }));
            }
        }

        private ICommand _ClearCoptersCommand;
        public ICommand ClearCoptersCommand
        {
            get
            {
                return _ClearCoptersCommand ?? (_ClearCoptersCommand = new RelayCommand(async () =>
                {
                    foreach (var copter in _copterManager.Copters)
                    {
                        await copter.DisconnectAsync();
                    }
                    await Task.Delay(100);  // 如果不等待一段时间，很可能会再触发 DataStreamReceived 事件导致飞行器重新出现在地图上。
                    _copterManager.Copters.Clear();
                    _copterManager.CopterStatus.Clear();
                    _mapManager.ClearCopters();
                    _flightTaskManager.ClearTasks();
                    UdpServerConnectionManager.Instance.ClearConnections();
                    _virtualCopterId = 1;
                    _lastVirtualCopterLocation = null;
                }));
            }
        }



    }
}
