﻿using Plane.Communication;
using Plane.FormationCreator.Formation;
using Plane.Windows.Messages;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.CommandWpf;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;

namespace Plane.FormationCreator.ViewModels
{
    public class MainViewModel : ViewModelBase
    {
        public MainViewModel(CopterListViewModel copterListViewModel)
        {
            _copterListViewModel = copterListViewModel;

            Plane.Windows.Messages.Message.Configure(showAction: msg => this.Message = msg);

            this.SwitchVelocityModeButtonContent = GetSwitchVelocityModeButtonContent();

            AppEx.Current.PropertyChanged += AppEx_PropertyChanged;
        }

        private CopterListViewModel _copterListViewModel;
        public CopterListViewModel CopterListViewModel { get { return _copterListViewModel; } }

        public AppEx AppEx { get; } = AppEx.Current;

        private void AppEx_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case nameof(AppEx.IsInFastMode):
                    this.SwitchVelocityModeButtonContent = GetSwitchVelocityModeButtonContent();
                    break;
                default:
                    break;
            }
        }

        private string _SwitchVelocityModeButtonContent;
        public string SwitchVelocityModeButtonContent
        {
            get { return _SwitchVelocityModeButtonContent; }
            set { Set(nameof(SwitchVelocityModeButtonContent), ref _SwitchVelocityModeButtonContent, value); }
        }

        private string _Message;
        public string Message
        {
            get { return _Message; }
            set { Set(nameof(Message), ref _Message, value); }
        }

        private ICommand _RestartListeningCommand;
        public ICommand RestartListeningCommand
        {
            get
            {
                return _RestartListeningCommand ?? (_RestartListeningCommand = new RelayCommand(() =>
                {
                    TcpServerConnectionManager.Instance.StopListening();
                    if (TcpServerConnectionManager.Instance.StartListening())
                    {
                        Alert.Show("已重启对连接请求的监听。");
                    }
                    else
                    {
                        Alert.Show("网络连接不正常，无法启动监听。");
                    }

                    UdpServerConnectionManager.Instance.StopReceiving();
                    UdpServerConnectionManager.Instance.StartReceiving();
                }));
            }
        }

        private ICommand _SwitchVelocityModeCommand;
        public ICommand SwitchVelocityModeCommand
        {
            get
            {
                return _SwitchVelocityModeCommand ?? (_SwitchVelocityModeCommand = new RelayCommand(() =>
                {
                    AppEx.Current.IsInFastMode = !AppEx.Current.IsInFastMode;
                }));
            }
        }

        private ICommand _SwitchAppModeCommand;
        public ICommand SwitchAppModeCommand
        {
            get
            {
                return _SwitchAppModeCommand ?? (_SwitchAppModeCommand = new RelayCommand<AppMode>(mode =>
                {
                    AppEx.Current.AppMode = mode;
                }));
            }
        }

        private ICommand _ShowOrHideModifyTaskViewCommand;
        public ICommand ShowOrHideModifyTaskViewCommand
        {
            get
            {
                return _ShowOrHideModifyTaskViewCommand ?? (_ShowOrHideModifyTaskViewCommand = new RelayCommand(() =>
                {
                    AppEx.Current.ShowModifyTaskView = !AppEx.Current.ShowModifyTaskView;
                }));
            }
        }

        private string GetSwitchVelocityModeButtonContent()
            => AppEx.Current.IsInFastMode ? "高速模式" : "低速模式";
    }
}
